from flask import flash, redirect, render_template, url_for
from flask_login import current_user, login_required
from app import db
from app.models import UserSettingJobStatusMailNotificationLevel
from . import bp
from .forms import ChangePasswordForm, EditNotificationSettingsForm


@bp.route('', methods=['GET', 'POST'])
@login_required
def settings():
    change_password_form = ChangePasswordForm(
        current_user,
        prefix='change-password-form'
    )
    edit_notification_settings_form = EditNotificationSettingsForm(
        data=current_user.to_json_serializeable(),
        prefix='edit-notification-settings-form'
    )
    # region handle change_password_form POST
    if change_password_form.submit.data and change_password_form.validate():
        current_user.password = change_password_form.new_password.data
        db.session.commit()
        flash('Your changes have been saved')
        return redirect(url_for('.settings'))
    # endregion handle change_password_form POST
    # region handle edit_notification_settings_form POST
    if edit_notification_settings_form.submit and edit_notification_settings_form.validate():
        current_user.setting_job_status_mail_notification_level = edit_notification_settings_form.job_status_mail_notification_level.data
        db.session.commit()
        flash('Your changes have been saved')
        return redirect(url_for('.settings'))
    # endregion handle edit_notification_settings_form POST
    return render_template(
        'settings/settings.html.j2',
        change_password_form=change_password_form,
        edit_notification_settings_form=edit_notification_settings_form,
        title='Settings'
    )
